use strict;
use warnings;

# this test was generated with Dist::Zilla::Plugin::Test::NoTabs 0.15

use Test::More 0.88;
use Test::NoTabs;

my @files = (
    'bin/sql-split',
    'lib/SQL/SplitStatement.pm',
    't/00-compile.t',
    't/00-load.t',
    't/00-report-prereqs.dd',
    't/00-report-prereqs.t',
    't/05-empty_statements.t',
    't/10-simple.t',
    't/20-hard.t',
    't/25-keywords_as_unquoted_identifiers.t',
    't/30-nested_blocks.t',
    't/40-comments.t',
    't/42-slash_terminates.t',
    't/50-transactions.t',
    't/51-transactions_and_proc.t',
    't/60-placeholders.t',
    't/61-placeholders_empty.t',
    't/70-proc_plsql_mixed.t',
    't/70-proc_plsql_slash_endings.t',
    't/71-proc_plsql_RTbug_57971.t',
    't/72-proc_plsql_alias.t',
    't/72-proc_plsql_function_inside_package.t',
    't/72-proc_plsql_mixed_endings.t',
    't/72-proc_plsql_nested_functions.t',
    't/72-proc_plsql_package.t',
    't/74-proc_plpgsql_dollar_quoted.t',
    't/74-proc_plpgsql_dollar_quoted_w_transactions.t',
    't/75-proc_grant.t',
    't/80-examples.t',
    't/81-examples_procedural.t',
    't/82-synopsis.t',
    't/90-mysql_delimiter.t',
    't/90-mysql_mixed.t',
    't/90-mysql_sakila.t',
    't/92-postgresql_pagila.t',
    't/94-exception.t',
    't/95-script.t',
    't/data/create_table.sql',
    't/data/create_table_and_trigger.sql',
    't/data/pagila-schema.sql',
    't/data/sakila-schema.sql'
);

notabs_ok($_) foreach @files;
done_testing;
